<?php
/**
 * Corretor CRM - https://app.corretorcrm.com
 *
 * @version   1.4.1

 * @copyright Copyright (c) 2022-2024 APPCRM DIGITAL TEC
 */

namespace App\Installer;

class PrivilegesChecker
{
    /**
     * Initialize new PrivilegesChecker instance.
     */
    public function __construct(protected DatabaseTest $tester)
    {
    }

    /**
     * Check the privileges.
     *
     * @throws \App\Installer\PrivilegeNotGrantedException
     */
    public function check(): void
    {
        $testMethods = $this->getTesterMethods();

        foreach ($testMethods as $test) {
            $this->tester->{$test}();

            throw_if(
                $this->tester->getLastError(),
                new PrivilegeNotGrantedException($this->tester->getLastError())
            );
        }
    }

    /**
     * Get the tester methods.
     */
    public static function getTesterMethods(): array
    {
        return [
            'testDropTable', // Should be first, it's the most important because all other tests are dropping the table.
            'testCreateTable',
            'testSelect',
            'testInsert',
            'testUpdate',
            'testDelete',
            'testAlter',
            'testIndex',
            'testReferences',
        ];
    }
}
